<?php

namespace Drupal\freecurrency\Services;

use Exception;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\ConnectException;
use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Exception\ServerException;
use GuzzleHttp\Exception\TooManyRedirectsException;

/**
 * FreecurrencyAPI Client.
 */
class FreecurrencyAPIClient {

  public const URL_STATUS = 'https://api.freecurrencyapi.com/v1/status';
  public const URL_GET_CURRENCIES = 'https://api.freecurrencyapi.com/v1/currencies';
  public const URL_GET_RATES = 'https://api.freecurrencyapi.com/v1/latest';

  /**
   * Function for make Request.
   *
   * @return array
   */
  public static function request($url, $api_key){
    try {
      $client = new Client();
      $response = $client->get($url . '?apikey=' . $api_key);
      if ($response->getStatusCode() === 200) {
        $json = $response->getBody()->getContents();
        $data = json_decode($json, TRUE);
        if ($data) {
          return ['state' => 'ok', 'data' => json_decode($json, TRUE)];
        } else {
          return ['state' => 'error', 'data' => 'JSON parse error'];
        }
      } else {
        return ['state' => 'error', 'data' => 'state: ' . $response->getStatusCode()];
      }
    } catch (ClientException $e) {
      return ['state' => 'error', 'data' => 'client exception (4xx errors): ' . $e->getMessage()];
    } catch (ServerException $e) {
      return ['state' => 'error', 'data' => 'server exception (5xx errors): ' . $e->getMessage()];
    } catch (RequestException $e) {
      return ['state' => 'error', 'data' => 'request exception: ' . $e->getMessage()];
    } catch (ConnectException $e) {
      return ['state' => 'error', 'data' => 'connect exception: ' . $e->getMessage()];
    } catch (TooManyRedirectsException $e) {
      return ['state' => 'error', 'data' => 'too many redirects exception: ' . $e->getMessage()];
    } catch (Exception $e) {
      return ['state' => 'error', 'data' => 'exception: ' . $e->getMessage()];
    }
  }

  /**
   * Function for get Service Status.
   *
   * @return array
   */
  public static function getStatus($api_key){
    $result = static::request(static::URL_STATUS, $api_key);
    if (isset($result['state']) && $result['state'] === 'ok') {
      return true;
    }
  }

  /**
   * Function for get Currencies.
   *
   * @return array
   */
  public static function getCurrencies($api_key){
    $result = static::request(static::URL_GET_CURRENCIES, $api_key);
    if (isset($result['state']) && $result['state'] === 'ok' && !empty($result['data']['data'])) {
      return ['state' => 'ok', 'data' => $result['data']['data']];
    }
    return $result;
  }

  /**
   * Function for get Rates.
   *
   * @return array
   */
  public static function getRates($api_key){
    $result = static::request(static::URL_GET_RATES, $api_key);
    if (isset($result['state']) && $result['state'] === 'ok' && !empty($result['data']['data'])) {
      return ['state' => 'ok', 'data' => $result['data']['data']];
    }
    return $result;
  }

}
